@@ -1,21 +1,27 @@
-use std::rc::Rc;
 use std::borrow::Borrow;
-use std::path::{Path, PathBuf};
+use std::path::Path;
 use std::num::NonZeroU32;
-use polodb_bson::{Document, Value, ObjectIdMaker, mk_document};
-use super::page::{header_page_wrapper, PageHandler};
+use std::sync::Arc;
+use std::rc::Rc;
+use polodb_bson::{Document, Value, ObjectIdMaker, doc};
+use super::page::header_page_wrapper;
 use super::error::DbErr;
+use super::TransactionType;
+use crate::page_handler::PageHandler;
 use crate::Config;
 use crate::vm::{SubProgram, VM, VmState};
 use crate::db::DbResult;
 use crate::meta_doc_helper::{meta_doc_key, MetaDocEntry};
 use crate::index_ctx::{IndexCtx, merge_options_into_default};
 use crate::btree::*;
-use crate::page::{RawPage, TransactionState};
+use crate::transaction::TransactionState;
+use crate::backend::file::FileBackend;
+use crate::backend::memory::MemoryBackend;
+use crate::page::RawPage;
 use crate::db_handle::DbHandle;
-use crate::journal::TransactionType;
 use crate::dump::{FullDump, PageDump, OverflowDataPageDump, DataPageDump, FreeListPageDump, BTreePageDump};
 use crate::page::header_page_wrapper::HeaderPageWrapper;
+use crate::backend::Backend;
 
 macro_rules! try_multiple {
     ($err: expr, $action: expr) => {
@@ -54,10 +60,10 @@ fn index_already_exists(index_doc: &Document, key: &str) -> bool {
  * API for all platforms
  */
 pub struct DbContext {
-    path:                PathBuf,
-    page_handler:        Box<PageHandler>,
-    obj_id_maker:        ObjectIdMaker,
-    meta_version:        u32,
+    page_handler: Box<PageHandler>,
+    obj_id_maker: ObjectIdMaker,
+    pub(crate)meta_version: u32,
+    config:       Arc<Config>,
 
 }
 
@@ -76,19 +82,32 @@ pub struct CollectionMeta {
 
 impl DbContext {
 
-    pub fn new(path: &Path, config: Config) -> DbResult<DbContext> {
+    pub fn open_file(path: &Path, config: Config) -> DbResult<DbContext> {
         let page_size = NonZeroU32::new(4096).unwrap();
 
-        let page_handler = PageHandler::with_config(path, page_size, Rc::new(config))?;
+        let config = Arc::new(config);
+        let backend = Box::new(FileBackend::open(path, page_size, config.clone())?);
+        DbContext::open_with_backend(backend, page_size, config)
+    }
+
+    pub fn open_memory(config: Config) -> DbResult<DbContext> {
+        let page_size = NonZeroU32::new(4096).unwrap();
+        let config = Arc::new(config);
+        let backend = Box::new(MemoryBackend::new(page_size, config.init_block_count));
+        DbContext::open_with_backend(backend, page_size, config)
+    }
+
+    fn open_with_backend(backend: Box<dyn Backend + Send>, page_size: NonZeroU32, config: Arc<Config>) -> DbResult<DbContext> {
+        let page_handler = PageHandler::new(backend, page_size, config.clone())?;
 
         let obj_id_maker = ObjectIdMaker::new();
 
         let mut ctx = DbContext {
-            path: path.to_path_buf(),
             page_handler: Box::new(page_handler),
             // first_page,
             obj_id_maker,
             meta_version: 0,
+            config,
         };
 
         let meta_source = ctx.get_meta_source()?;
@@ -127,11 +146,11 @@ impl DbContext {
 
         let collection_meta = MetaDocEntry::new(0, "<meta>".into(), meta_src.meta_pid);
 
-        let query_doc = mk_document! {
+        let query_doc = doc! {
             "name": name,
         };
 
-        let meta_doc = mk_document!{};
+        let meta_doc = doc!{};
 
         let subprogram = SubProgram::compile_query(
             &collection_meta,
@@ -196,11 +215,11 @@ impl DbContext {
     fn check_collection_exist(&mut self, name: &str, meta_src: &MetaSource) -> DbResult<bool> {
         let collection_meta = MetaDocEntry::new(0, "<meta>".into(), meta_src.meta_pid);
 
-        let query_doc = mk_document! {
+        let query_doc = doc! {
             "name": name,
         };
 
-        let meta_doc = mk_document!{};
+        let meta_doc = doc!{};
 
         let subprogram = SubProgram::compile_query(
             &collection_meta,
@@ -248,6 +267,10 @@ impl DbContext {
 
         let insert_result = btree_wrapper.insert_item(&doc, false)?;
 
+        // if a backward item returns, it's saying that the btree has been "rotated".
+        // the center node of the btree has been changed.
+        // So you have to distribute a new page to store the "central node",
+        // and the newer page is the center of the btree.
         if let Some(backward_item) = insert_result.backward_item {
             let new_root_id = self.page_handler.alloc_page_id()?;
 
@@ -416,7 +439,8 @@ impl DbContext {
         let mut is_meta_changed = false;
 
         // insert index begin
-        let mut index_ctx_opt = IndexCtx::from_meta_doc(collection_meta.doc_ref());
+        let serialize_type = self.config.serialize_type;
+        let mut index_ctx_opt = IndexCtx::from_meta_doc(collection_meta.doc_ref(), serialize_type);
         if let Some(index_ctx) = &mut index_ctx_opt {
             let mut is_ctx_changed = false;
 
@@ -510,7 +534,7 @@ impl DbContext {
         Ok(vm.r2 as usize)
     }
 
-    pub fn drop(&mut self, col_id: u32, meta_version: u32) -> DbResult<()> {
+    pub fn drop_collection(&mut self, col_id: u32, meta_version: u32) -> DbResult<()> {
         self.check_meta_version(meta_version)?;
 
         self.page_handler.auto_start_transaction(TransactionType::Write)?;
@@ -526,8 +550,9 @@ impl DbContext {
             0, meta_source.meta_pid, col_id)?;
         delete_all_helper::delete_all(&mut self.page_handler, collection_meta)?;
 
+        let serialize_type = self.config.serialize_type;
         let mut btree_wrapper = BTreePageDeleteWrapper::new(
-            &mut self.page_handler, meta_source.meta_pid);
+            &mut self.page_handler, meta_source.meta_pid, serialize_type);
 
         let pkey = Value::from(col_id);
         btree_wrapper.delete_item(&pkey)?;
@@ -647,15 +672,17 @@ impl DbContext {
         let collection_meta = self.find_collection_root_pid_by_id(
             0, meta_source.meta_pid, col_id)?;
 
+        let serialize_type = self.config.serialize_type;
         let mut delete_wrapper = BTreePageDeleteWrapper::new(
             &mut self.page_handler,
-            collection_meta.root_pid() as u32
+            collection_meta.root_pid() as u32,
+            serialize_type
         );
         let result = delete_wrapper.delete_item(key)?;
         delete_wrapper.flush_pages()?;
 
         if let Some(deleted_item) = &result {
-            let index_ctx_opt = IndexCtx::from_meta_doc(collection_meta.doc_ref());
+            let index_ctx_opt = IndexCtx::from_meta_doc(collection_meta.doc_ref(), serialize_type);
             if let Some(index_ctx) = &index_ctx_opt {
                 index_ctx.delete_index_by_content(deleted_item.borrow(), &mut self.page_handler)?;
             }
@@ -674,22 +701,29 @@ impl DbContext {
         counter_helper::count(&mut self.page_handler, collection_meta)
     }
 
-    pub fn query_all_meta(&mut self) -> DbResult<Vec<Rc<Document>>> {
-        // let meta_page_id = self.get_meta_page_id()?;
-        //
-        // let mut result = vec![];
-        // let mut cursor = Cursor::new(&mut self.page_handler, meta_page_id)?;
-        //
-        // while cursor.has_next() {
-        //     let ticket = cursor.peek().unwrap();
-        //     let doc = cursor.get_doc_from_ticket(&ticket)?;
-        //     result.push(doc);
-        //
-        //     let _ = cursor.next()?;
-        // }
-        //
-        // Ok(result)
-        unimplemented!()
+    pub(crate) fn query_all_meta(&mut self) -> DbResult<Vec<Rc<Document>>> {
+        let meta_src = self.get_meta_source()?;
+
+        let collection_meta = MetaDocEntry::new(0, "<meta>".into(), meta_src.meta_pid);
+
+        let subprogram = SubProgram::compile_query_all(
+            &collection_meta,
+            true)?;
+
+        let mut handle = self.make_handle(subprogram);
+        handle.step()?;
+
+        let mut result = vec![];
+
+        while handle.state() == (VmState::HasRow as i8) {
+            let doc = handle.get().unwrap_document();
+
+            result.push(doc.clone());
+
+            handle.step()?;
+        }
+
+        Ok(result)
     }
 
     pub fn start_transaction(&mut self, ty: Option<TransactionType>) -> DbResult<()> {
@@ -726,7 +760,6 @@ impl DbContext {
     }
 
     pub fn dump(&mut self) -> DbResult<FullDump> {
-        let file_meta = self.page_handler.file_meta()?;
         let first_page = self.page_handler.pipeline_read_page(0)?;
         let first_page_wrapper = HeaderPageWrapper::from_raw_page(first_page);
         let version = first_page_wrapper.get_version();
@@ -735,23 +768,21 @@ impl DbContext {
         let free_list_size = first_page_wrapper.get_free_list_size();
         let page_size = self.page_handler.page_size;
 
-        let pages = self.dump_all_pages(file_meta.len())?;
         let journal_dump = self.page_handler.dump_journal()?;
         let full_dump = FullDump {
-            path: self.path.clone(),
             identifier: first_page_wrapper.get_title(),
             version: dump_version(&version),
-            file_meta,
             journal_dump,
             meta_pid,
             free_list_pid,
             free_list_size,
             page_size,
-            pages,
+            pages: vec![],
         };
         Ok(full_dump)
     }
 
+    #[allow(dead_code)]
     fn dump_all_pages(&mut self, file_len: u64) -> DbResult<Vec<PageDump>> {
         let page_count = file_len / (self.page_handler.page_size.get() as u64);
         let mut result = Vec::with_capacity(page_count as usize);
@@ -771,6 +802,7 @@ impl DbContext {
 
 }
 
+#[allow(dead_code)]
 fn dump_page(raw_page: RawPage) -> DbResult<PageDump> {
     let first = raw_page.data[0];
     let second = raw_page.data[1];
@@ -812,11 +844,6 @@ impl Drop for DbContext {
         if self.page_handler.transaction_state() != TransactionState::NoTrans {
             let _ = self.page_handler.only_rollback_journal();
         }
-        let checkpoint_result = self.page_handler.checkpoint_journal();  // ignored
-        if checkpoint_result.is_ok() {
-            let path = self.page_handler.journal_file_path().to_path_buf();
-            let _ = std::fs::remove_file(path);  // ignore the result
-        }
     }
 
 }
