@@ -1,12 +1,19 @@
+use std::convert::TryFrom;
 use std::rc::Rc;
 use std::path::Path;
-use polodb_bson::{Document, ObjectId};
+use std::io::{Read, Write};
+use std::sync::atomic::{AtomicBool, Ordering};
+use polodb_bson::{Document, ObjectId, Value};
+use byteorder::{self, BigEndian, ReadBytesExt, WriteBytesExt};
 use super::error::DbErr;
+use crate::msg_ty::MsgTy;
 use crate::Config;
 use crate::context::DbContext;
 use crate::{DbHandle, TransactionType};
 use crate::dump::FullDump;
 
+pub(crate) static SHOULD_LOG: AtomicBool = AtomicBool::new(false);
+
 fn consume_handle_to_vec(handle: &mut DbHandle, result: &mut Vec<Rc<Document>>) -> DbResult<()> {
     handle.step()?;
 
@@ -20,6 +27,15 @@ fn consume_handle_to_vec(handle: &mut DbHandle, result: &mut Vec<Rc<Document>>)
     Ok(())
 }
 
+macro_rules! unwrap_str_or {
+    ($expr: expr, $or: expr) => {
+        match $expr {
+            Some(Value::String(id)) => id.as_str(),
+            _ => return Err(DbErr::ParseError($or)),
+        }
+    }
+}
+
 /// A wrapper of collection in struct.
 ///
 /// All CURD methods can be done through this structure.
@@ -51,11 +67,11 @@ fn consume_handle_to_vec(handle: &mut DbHandle, result: &mut Vec<Rc<Document>>)
 /// ```rust
 /// use std::rc::Rc;
 /// use polodb_core::Database;
-/// use polodb_bson::mk_document;
+/// use polodb_bson::doc;
 ///
-/// let mut db = Database::open("/tmp/test-collection").unwrap();
+/// let mut db = Database::open_file("/tmp/test-collection").unwrap();
 /// let mut collection = db.collection("test").unwrap();
-/// collection.insert(mk_document! {
+/// collection.insert(doc! {
 ///     "_id": 0,
 ///     "name": "Vincent Chan",
 ///     "score": 99.99,
@@ -152,6 +168,8 @@ impl<'a>  Collection<'a> {
         self.db.ctx.update(self.id, self.meta_version, query, update)
     }
 
+    /// Insert a document into the database.
+    /// The returning boolean value represents if the DB inserted a "_id" for you.
     #[inline]
     pub fn insert(&mut self, doc: &mut Document) -> DbResult<bool> {
         self.db.ctx.insert(self.id, self.meta_version, doc)
@@ -170,7 +188,7 @@ impl<'a>  Collection<'a> {
         }
     }
 
-    // // release in 0.2
+    /// release in 0.12
     #[allow(dead_code)]
     fn create_index(&mut self, keys: &Document, options: Option<&Document>) -> DbResult<()> {
         self.db.ctx.create_index(self.id, keys, options)
@@ -210,7 +228,7 @@ impl<'a>  Collection<'a> {
 /// ```rust
 /// use polodb_core::Database;
 ///
-/// let mut db = Database::open("/tmp/test-polo.db").unwrap();
+/// let mut db = Database::open_file("/tmp/test-polo.db").unwrap();
 /// let test_collection = db.collection("test").unwrap();
 /// ```
 pub struct Database {
@@ -221,17 +239,45 @@ pub type DbResult<T> = Result<T, DbErr>;
 
 impl Database {
 
+    #[inline]
+    pub fn set_log(v: bool) {
+        SHOULD_LOG.store(v, Ordering::SeqCst);
+    }
+
     #[inline]
     pub fn mk_object_id(&mut self) -> ObjectId {
         self.ctx.object_id_maker().mk_object_id()
     }
 
+    #[deprecated]
     pub fn open<P: AsRef<Path>>(path: P) -> DbResult<Database>  {
-        Database::open_with_config(path, Config::default())
+        Database::open_file(path)
     }
 
+    #[deprecated]
     pub fn open_with_config<P: AsRef<Path>>(path: P, config: Config) -> DbResult<Database>  {
-        let ctx = DbContext::new(path.as_ref(), config)?;
+        Database::open_file_with_config(path, config)
+    }
+
+    pub fn open_file<P: AsRef<Path>>(path: P) -> DbResult<Database>  {
+        Database::open_file_with_config(path, Config::default())
+    }
+
+    pub fn open_file_with_config<P: AsRef<Path>>(path: P, config: Config) -> DbResult<Database>  {
+        let ctx = DbContext::open_file(path.as_ref(), config)?;
+        let rc_ctx = Box::new(ctx);
+
+        Ok(Database {
+            ctx: rc_ctx,
+        })
+    }
+
+    pub fn open_memory() -> DbResult<Database> {
+        Database::open_memory_wht_config(Config::default())
+    }
+
+    pub fn open_memory_wht_config(config: Config) -> DbResult<Database> {
+        let ctx = DbContext::open_memory(config)?;
         let rc_ctx = Box::new(ctx);
 
         Ok(Database {
@@ -304,13 +350,367 @@ impl Database {
         self.ctx.query_all_meta()
     }
 
+    /// Upgrade DB from v1 to v2
+    /// The older file will be renamed as (name).old
+    pub fn v1_to_v2(path: &Path) -> DbResult<()> {
+        crate::migration::v1_to_v2(path)
+    }
+
+    /// handle request for database
+    /// See [MsgTy] for message detail
+    pub fn handle_request<R: Read, W: Write>(&mut self, pipe_in: &mut R, pipe_out: &mut W) -> std::io::Result<MsgTy> {
+        let mut buffer: Vec<u8> = Vec::new();
+        let result = self.handle_request_with_result(pipe_in, &mut buffer);
+        let ret = match &result {
+            Ok(t) => t.clone(),
+            Err(_) => MsgTy::Undefined,
+        };
+        if let Err(DbErr::IOErr(io_err)) = result {
+            return Err(*io_err);
+        }
+        let resp_result = self.send_response_with_result(pipe_out, result, buffer);
+        if let Err(DbErr::IOErr(io_err)) = resp_result {
+            return Err(*io_err);
+        }
+        if let Err(err) = resp_result {
+            eprintln!("resp error: {}", err);
+        }
+        Ok(ret)
+    }
+
+    fn send_response_with_result<W: Write>(&mut self, pipe_out: &mut W, result: DbResult<MsgTy>, body: Vec<u8>) -> DbResult<()> {
+        match result {
+            Ok(msg_ty) => {
+                let val = msg_ty as i32;
+                pipe_out.write_i32::<BigEndian>(val)?;
+                pipe_out.write_u32::<BigEndian>(body.len() as u32)?;
+                pipe_out.write(&body)?;
+            }
+
+            Err(err) => {
+                pipe_out.write_i32::<BigEndian>(-1)?;
+                let str = format!("resp with error: {}", err);
+                let str_buf = str.as_bytes();
+                pipe_out.write_u32::<BigEndian>(str_buf.len() as u32)?;
+                pipe_out.write(str_buf)?;
+            }
+        }
+        Ok(())
+    }
+
+    fn handle_request_with_result<R: Read, W: Write>(&mut self, pipe_in: &mut R, pipe_out: &mut W) -> DbResult<MsgTy> {
+        let msg_ty_int = pipe_in.read_i32::<BigEndian>()?;
+
+        let msg_ty = MsgTy::try_from(msg_ty_int)?;
+
+        match msg_ty {
+            MsgTy::Find => {
+                self.handle_find_operation(pipe_in, pipe_out)?;
+            },
+
+            MsgTy::FindOne => {
+                self.handle_find_one_operation(pipe_in, pipe_out)?;
+            },
+
+            MsgTy::Insert => {
+                self.handle_insert_operation(pipe_in, pipe_out)?;
+            }
+
+            MsgTy::Update => {
+                self.handle_update_operation(pipe_in, pipe_out)?;
+            }
+
+            MsgTy::Delete => {
+                self.handle_delete_operation(pipe_in, pipe_out)?;
+            }
+
+            MsgTy::CreateCollection => {
+                self.handle_create_collection(pipe_in, pipe_out)?;
+            }
+
+            MsgTy::Drop =>{
+                self.handle_drop_collection(pipe_in, pipe_out)?;
+            }
+
+            MsgTy::StartTransaction => {
+                self.handle_start_transaction(pipe_in, pipe_out)?;
+            }
+
+            MsgTy::Commit => {
+                self.handle_commit(pipe_in, pipe_out)?;
+            }
+
+            MsgTy::Rollback => {
+                self.handle_rollback(pipe_in, pipe_out)?;
+            }
+
+            MsgTy::Count => {
+                self.handle_count_operation(pipe_in, pipe_out)?;
+            }
+
+            MsgTy::SafelyQuit => (),
+
+            _ => {
+                return Err(DbErr::ParseError("unknown msg type".into()));
+            }
+
+        };
+
+        Ok(msg_ty)
+    }
+
+    fn handle_start_transaction<R: Read, W: Write>(&mut self, pipe_in: &mut R, _pipe_out: &mut W) -> DbResult<()> {
+        let value = self.receive_request_body(pipe_in)?;
+        let transaction_type = match value {
+            Value::Int(val) => val,
+            _ => {
+                return Err(DbErr::ParseError("transaction needs a type".into()));
+            }
+        };
+        match transaction_type {
+            0 => self.start_transaction(None),
+            1 => self.start_transaction(Some(TransactionType::Read)),
+            2 => self.start_transaction(Some(TransactionType::Write)),
+            _ => return Err(DbErr::ParseError("invalid transaction type".into())),
+        }
+    }
+
+    fn handle_commit<R: Read, W: Write>(&mut self, pipe_in: &mut R, _pipe_out: &mut W) -> DbResult<()> {
+        let _ = self.receive_request_body(pipe_in)?;
+        self.commit()?;
+        Ok(())
+    }
+
+    fn handle_rollback<R: Read, W: Write>(&mut self, pipe_in: &mut R, _pipe_out: &mut W) -> DbResult<()> {
+        let _ = self.receive_request_body(pipe_in)?;
+        self.rollback()?;
+        Ok(())
+    }
+
+    fn handle_find_one_operation<R: Read, W: Write>(&mut self, pipe_in: &mut R, pipe_out: &mut W) -> DbResult<()> {
+        let value = self.receive_request_body(pipe_in)?;
+
+        let doc = match value {
+            Value::Document(doc) => doc,
+            _ => return Err(DbErr::ParseError(format!("value is not a doc in find one request, actual: {}", value))),
+        };
+
+        let collection_name: &str = unwrap_str_or!(doc.get("cl"), "cl not found in find request".into());
+
+        let mut query_opt = match doc.get("query") {
+            Some(Value::Document(doc)) => doc.clone(),
+            _ => return Err(DbErr::ParseError("query not found in find request".into())),
+        };
+
+        let mut_doc = Rc::make_mut(&mut query_opt);
+        let mut collection = self.collection(collection_name)?;
+
+        let result = collection.find_one(mut_doc)?;
+
+        let result_value = match result {
+            Some(doc) => Value::Document(doc),
+            None => Value::Null,
+        };
+
+        result_value.to_msgpack(pipe_out)?;
+
+        Ok(())
+    }
+
+    fn receive_request_body<R: Read>(&mut self, pipe_in: &mut R) -> DbResult<Value> {
+        let request_size = pipe_in.read_u32::<BigEndian>()? as usize;
+        if request_size == 0 {
+            return Ok(Value::Null);
+        }
+        let mut request_body = vec![0u8; request_size];
+        pipe_in.read_exact(&mut request_body)?;
+        let mut body_ref: &[u8] = request_body.as_slice();
+        let val = Value::from_msgpack(&mut body_ref)?;
+        Ok(val)
+    }
+
+    fn handle_find_operation<R: Read, W: Write>(&mut self, pipe_in: &mut R, pipe_out: &mut W) -> DbResult<()> {
+        let value = self.receive_request_body(pipe_in)?;
+
+        let doc = match value {
+            Value::Document(doc) => doc,
+            _ => return Err(DbErr::ParseError(format!("value is not a doc in find request, actual: {}", value))),
+        };
+
+        let collection_name: &str = unwrap_str_or!(doc.get("cl"), "cl not found in find request".into());
+
+        let query_opt = match doc.get("query") {
+            Some(Value::Document(doc)) => Some(doc),
+            _ => None,
+        };
+
+        let mut collection = self.collection(collection_name)?;
+
+        let result = if let Some(query) = query_opt {
+            collection.find(query)?
+        } else {
+            collection.find_all()?
+        };
+
+        let mut value_arr = polodb_bson::Array::new();
+
+        for item in result {
+            value_arr.push(Value::Document(item));
+        }
+
+        let result_value = Value::Array(Rc::new(value_arr));
+        result_value.to_msgpack(pipe_out)?;
+
+        Ok(())
+    }
+
+    fn handle_insert_operation<R: Read, W: Write>(&mut self, pipe_in: &mut R, pipe_out: &mut W) -> DbResult<()> {
+        let value = self.receive_request_body(pipe_in)?;
+
+        let doc = match value {
+            Value::Document(doc) => doc,
+            _ => return Err(DbErr::ParseError(format!("value is not a doc in insert request, actual: {}", value))),
+        };
+
+        let collection_name: &str = unwrap_str_or!(doc.get("cl"), "cl not found in find request".into());
+
+        let mut insert_data = match doc.get("data") {
+            Some(Value::Document(doc)) => doc.clone(),
+            _ => return Err(DbErr::ParseError("query not found in insert request".into())),
+        };
+
+        let mut_doc = Rc::make_mut(&mut insert_data);
+
+        let mut collection = self.collection(collection_name)?;
+        let id_changed = collection.insert(mut_doc)?;
+
+        let ret_value = if id_changed {
+            mut_doc.get("_id").unwrap().clone()
+        } else {
+            Value::Null
+        };
+
+        ret_value.to_msgpack(pipe_out)?;
+
+        Ok(())
+    }
+
+    fn handle_update_operation<R: Read, W: Write>(&mut self, pipe_in: &mut R, pipe_out: &mut W) -> DbResult<()> {
+        let value = self.receive_request_body(pipe_in)?;
+
+        let doc = match value {
+            Value::Document(doc) => doc,
+            _ => return Err(DbErr::ParseError(format!("value is not a doc in update request, actual: {}", value))),
+        };
+
+        let collection_name: &str = unwrap_str_or!(doc.get("cl"), "cl not found in update request".into());
+
+        let query = match doc.get("query") {
+            Some(Value::Document(doc)) => Some(doc.as_ref()),
+            Some(_) => return Err(DbErr::ParseError("query is not a document in update request".into())),
+            None => None
+        };
+
+        let update_data = match doc.get("update") {
+            Some(Value::Document(doc)) => doc.clone(),
+            _ => return Err(DbErr::ParseError("'update' not found in update request".into())),
+        };
+
+        let mut collection = self.collection(collection_name)?;
+        let size = collection.update(query, update_data.as_ref())?;
+
+        let ret_val = Value::Int(size as i64);
+        ret_val.to_msgpack(pipe_out)?;
+
+        Ok(())
+    }
+
+    fn handle_delete_operation<R: Read, W: Write>(&mut self, pipe_in: &mut R, pipe_out: &mut W) -> DbResult<()> {
+        let value = self.receive_request_body(pipe_in)?;
+
+        let doc = match value {
+            Value::Document(doc) => doc,
+            _ => return Err(DbErr::ParseError(format!("value is not a doc in delete request, actual: {}", value))),
+        };
+
+        let collection_name: &str = unwrap_str_or!(doc.get("cl"), "cl not found in delete request".into());
+
+        let query = match doc.get("query") {
+            Some(Value::Document(doc)) => Some(doc.as_ref()),
+            Some(_) => return Err(DbErr::ParseError("query is not a document in delete request".into())),
+            None => None
+        };
+
+        let mut collection = self.collection(collection_name)?;
+        let size = collection.delete(query)?;
+
+        let ret_val = Value::Int(size as i64);
+        ret_val.to_msgpack(pipe_out)?;
+
+        Ok(())
+    }
+
+    fn handle_create_collection<R: Read, W: Write>(&mut self, pipe_in: &mut R, pipe_out: &mut W) -> DbResult<()> {
+        let value = self.receive_request_body(pipe_in)?;
+        let doc: Rc<Document> = match value {
+            Value::Document(d) => d,
+            _ => return Err(DbErr::ParseError(format!("create document expect a document, actual: {}", value))),
+        };
+        let name: String = match doc.get("name") {
+            Some(Value::String(s)) => s.as_str().into(),
+            _ => return Err(DbErr::ParseError(format!("should give the name of the collection to create"))),
+        };
+        let ret = match self.create_collection(&name) {
+            Ok(_) => Value::Boolean(true),
+            Err(DbErr::CollectionAlreadyExits(_)) => Value::Boolean(false),
+            Err(err) => return Err(err),
+        };
+        ret.to_msgpack(pipe_out)?;
+        Ok(())
+    }
+
+    fn handle_drop_collection<R: Read, W: Write>(&mut self, pipe_in: &mut R, _pipe_out: &mut W) -> DbResult<()> {
+        let value = self.receive_request_body(pipe_in)?;
+        let cl_name: String = match value {
+            Value::String(s) => s.as_str().into(),
+            _ => return Err(DbErr::ParseError(format!("should give the name of the collection to drop, actual: {}", value))),
+        };
+        let info = match self.ctx.get_collection_meta_by_name(&cl_name) {
+            Ok(meta) => meta,
+            Err(DbErr::CollectionNotFound(_)) => self.ctx.create_collection(&cl_name)?,
+            Err(err) => return Err(err),
+        };
+        self.ctx.drop_collection(info.id, info.meta_version)?;
+        Ok(())
+    }
+
+    fn handle_count_operation<R: Read, W: Write>(&mut self, pipe_in: &mut R, pipe_out: &mut W) -> DbResult<()> {
+        let value = self.receive_request_body(pipe_in)?;
+
+        let doc = match value {
+            Value::Document(doc) => doc,
+            _ => return Err(DbErr::ParseError(format!("value is not a doc in count request, actual: {}", value))),
+        };
+
+        let collection_name: &str = unwrap_str_or!(doc.get("cl"), "cl not found in count request".into());
+
+        let mut collection = self.collection(collection_name)?;
+
+        let count = collection.count()?;
+
+        let ret_val = Value::Int(count as i64);
+        ret_val.to_msgpack(pipe_out)?;
+
+        Ok(())
+    }
+
 }
 
 #[cfg(test)]
 mod tests {
     use std::rc::Rc;
     use std::env;
-    use polodb_bson::{Document, Value, mk_document};
+    use polodb_bson::{Document, Value, doc};
     use crate::{Database, Config, DbResult, DbErr};
     use std::io::Read;
     use std::path::PathBuf;
@@ -341,7 +741,7 @@ mod tests {
         let _ = std::fs::remove_file(db_path.as_path());
         let _ = std::fs::remove_file(journal_path);
 
-        Database::open_with_config(db_path.as_path().to_str().unwrap(), config)
+        Database::open_file_with_config(db_path.as_path().to_str().unwrap(), config)
     }
 
     fn prepare_db(db_name: &str) -> DbResult<Database> {
@@ -356,7 +756,7 @@ mod tests {
 
         for i in 0..size {
             let content = i.to_string();
-            let mut new_doc = mk_document! {
+            let mut new_doc = doc! {
                 "content": content,
             };
             collection.insert(&mut new_doc).unwrap();
@@ -372,7 +772,7 @@ mod tests {
         let mut test_collection = db.collection("test").unwrap();
         let all = test_collection.find_all( ).unwrap();
 
-        let second = test_collection.find_one(&mk_document! {
+        let second = test_collection.find_one(&doc! {
             "content": "1",
         }).unwrap().unwrap();
         assert_eq!(second.get("content").unwrap().unwrap_string(), "1");
@@ -383,19 +783,24 @@ mod tests {
 
     #[test]
     fn test_transaction_commit() {
-        let mut db = prepare_db("test-transaction").unwrap();
-        db.start_transaction(None).unwrap();
-        let mut collection = db.create_collection("test").unwrap();
+        vec![Some("test-transaction-commit"), None].iter().for_each(|value| {
+            let mut db = match value {
+                Some(name) => prepare_db(name).unwrap(),
+                None => Database::open_memory().unwrap()
+            };
+            db.start_transaction(None).unwrap();
+            let mut collection = db.create_collection("test").unwrap();
 
-        for i in 0..10{
-            let content = i.to_string();
-            let mut new_doc = mk_document! {
+            for i in 0..10{
+                let content = i.to_string();
+                let mut new_doc = doc! {
                     "_id": i,
                     "content": content,
                 };
-            collection.insert(&mut new_doc).unwrap();
-        }
-        db.commit().unwrap()
+                collection.insert(&mut new_doc).unwrap();
+            }
+            db.commit().unwrap()
+        });
     }
 
     #[test]
@@ -408,7 +813,7 @@ mod tests {
 
         for i in 0..1000 {
             let content = i.to_string();
-            let mut new_doc = mk_document! {
+            let mut new_doc = doc! {
                 "_id": i,
                 "content": content,
             };
@@ -420,7 +825,7 @@ mod tests {
         let mut collection2 = db.create_collection("test-2").unwrap();
         for i in 0..10{
             let content = i.to_string();
-            let mut new_doc = mk_document! {
+            let mut new_doc = doc! {
                 "_id": i,
                 "content": content,
             };
@@ -434,19 +839,19 @@ mod tests {
         let mut db = prepare_db("test-multiple-find-one").unwrap();
         {
             let mut collection = db.collection("config").unwrap();
-            let mut doc1 = mk_document! {
+            let mut doc1 = doc! {
                 "_id": "c1",
                 "value": "c1",
             };
             collection.insert(&mut doc1).unwrap();
 
-            let mut doc2 = mk_document! {
+            let mut doc2 = doc! {
                 "_id": "c2",
                 "value": "c2",
             };
             collection.insert(&mut doc2).unwrap();
 
-            let mut doc2 = mk_document! {
+            let mut doc2 = doc! {
                 "_id": "c3",
                 "value": "c3",
             };
@@ -457,31 +862,31 @@ mod tests {
 
         {
             let mut collection = db.collection("config").unwrap();
-            collection.update(Some(&mk_document! {
+            collection.update(Some(&doc! {
                 "_id": "c2",
-            }), &mk_document! {
-                "$set": mk_document! {
+            }), &doc! {
+                "$set": doc! {
                     "value": "c33",
                 },
             }).unwrap();
-            collection.update(Some(&mk_document! {
+            collection.update(Some(&doc! {
                 "_id": "c2",
-            }), &mk_document! {
-                "$set": mk_document! {
+            }), &doc! {
+                "$set": doc! {
                     "value": "c22",
                 },
             }).unwrap();
         }
 
         let mut collection = db.collection("config").unwrap();
-        let doc1 = collection.find_one(&mk_document! {
+        let doc1 = collection.find_one(&doc! {
             "_id": "c1",
         }).unwrap().unwrap();
 
         assert_eq!(doc1.get("value").unwrap().unwrap_string(), "c1");
 
         let mut collection = db.collection("config").unwrap();
-        let doc1 = collection.find_one(&mk_document! {
+        let doc1 = collection.find_one(&doc! {
             "_id": "c2",
         }).unwrap().unwrap();
 
@@ -490,28 +895,33 @@ mod tests {
 
     #[test]
     fn test_rollback() {
-        let mut db = prepare_db("test-rollback").unwrap();
-        let mut collection = db.create_collection("test").unwrap();
+        vec![Some("test-collection"), None].iter().for_each(|value| {
+            let mut db = match value {
+                Some(name) => prepare_db(name).unwrap(),
+                None => Database::open_memory().unwrap()
+            };
+            let mut collection = db.create_collection("test").unwrap();
 
-        assert_eq!(collection.count().unwrap(), 0);
+            assert_eq!(collection.count().unwrap(), 0);
 
-        db.start_transaction(None).unwrap();
+            db.start_transaction(None).unwrap();
 
-        let mut collection = db.collection("test").unwrap();
-        for i in 0..10{
-            let content = i.to_string();
-            let mut new_doc = mk_document! {
+            let mut collection = db.collection("test").unwrap();
+            for i in 0..10 {
+                let content = i.to_string();
+                let mut new_doc = doc! {
                 "_id": i,
                 "content": content,
             };
-            collection.insert(new_doc.as_mut()).unwrap();
-        }
-        assert_eq!(collection.count().unwrap(), 10);
+                collection.insert(new_doc.as_mut()).unwrap();
+            }
+            assert_eq!(collection.count().unwrap(), 10);
 
-        db.rollback().unwrap();
+            db.rollback().unwrap();
 
-        let mut collection = db.collection("test").unwrap();
-        assert_eq!(collection.count().unwrap(), 0);
+            let mut collection = db.collection("test").unwrap();
+            assert_eq!(collection.count().unwrap(), 0);
+        });
     }
 
     #[test]
@@ -522,7 +932,7 @@ mod tests {
 
             for i in 0..TEST_SIZE {
                 let content = i.to_string();
-                let mut new_doc = mk_document! {
+                let mut new_doc = doc! {
                     "_id": i,
                     "content": content,
                 };
@@ -548,7 +958,7 @@ mod tests {
         let mut collection = db.collection("test").unwrap();
 
         let result = collection.find(
-            &mk_document! {
+            &doc! {
                 "content": "3",
             }
         ).unwrap();
@@ -570,7 +980,7 @@ mod tests {
 
         let first_key = &all[0].pkey_id().unwrap();
 
-        let result = collection.find(&mk_document! {
+        let result = collection.find(&doc! {
             "_id": first_key.clone(),
         }).unwrap();
 
@@ -586,7 +996,7 @@ mod tests {
         {
             let mut db_path = env::temp_dir();
             db_path.push("test-reopen.db");
-            let _db2 = Database::open(db_path.as_path().to_str().unwrap()).unwrap();
+            let _db2 = Database::open_file(db_path.as_path().to_str().unwrap()).unwrap();
         }
     }
 
@@ -594,7 +1004,7 @@ mod tests {
     fn test_pkey_type_check() {
         let mut db = create_and_return_db_with_items("test-type-check", TEST_SIZE);
 
-        let mut doc = mk_document! {
+        let mut doc = doc! {
             "_id": 10,
             "value": "something",
         };
@@ -624,18 +1034,19 @@ mod tests {
     fn test_db_occupied() {
         const DB_NAME: &'static str = "test-db-lock";
         let db_path = mk_db_path(DB_NAME);
+        let _ = std::fs::remove_file(&db_path);
         {
             let config = Config::default();
-            let _db1 = Database::open_with_config(db_path.as_path().to_str().unwrap(), config).unwrap();
+            let _db1 = Database::open_file_with_config(db_path.as_path().to_str().unwrap(), config).unwrap();
             let config = Config::default();
-            let db2 = Database::open_with_config(db_path.as_path().to_str().unwrap(), config);
+            let db2 = Database::open_file_with_config(db_path.as_path().to_str().unwrap(), config);
             match db2 {
                 Err(DbErr::DatabaseOccupied) => assert!(true),
                 _ => assert!(false),
             }
         }
         let config = Config::default();
-        let _db3 = Database::open_with_config(db_path.as_path().to_str().unwrap(), config).unwrap();
+        let _db3 = Database::open_file_with_config(db_path.as_path().to_str().unwrap(), config).unwrap();
     }
 
     #[test]
@@ -643,7 +1054,7 @@ mod tests {
         let mut db = prepare_db("test-create-index").unwrap();
         let mut collection = db.create_collection("test").unwrap();
 
-        let keys = mk_document! {
+        let keys = doc! {
             "user_id": 1,
         };
 
@@ -651,14 +1062,14 @@ mod tests {
 
         for i in 0..10 {
             let str = Rc::new(i.to_string());
-            let mut data = mk_document! {
+            let mut data = doc! {
                 "name": str.clone(),
                 "user_id": str.clone(),
             };
             collection.insert(data.as_mut()).unwrap();
         }
 
-        let mut data = mk_document! {
+        let mut data = doc! {
             "name": "what",
             "user_id": 3,
         };
@@ -675,7 +1086,7 @@ mod tests {
         for i in 0..100 {
             let content = i.to_string();
 
-            let mut new_doc = mk_document! {
+            let mut new_doc = doc! {
                 "content": content,
             };
 
@@ -685,7 +1096,7 @@ mod tests {
 
         let third = &doc_collection[3];
         let third_key = third.get("_id").unwrap();
-        let delete_doc = mk_document! {
+        let delete_doc = doc! {
             "_id": third_key.clone(),
         };
         assert!(collection.delete(Some(&delete_doc)).unwrap() > 0);
@@ -701,7 +1112,7 @@ mod tests {
 
         for i in 0..1000 {
             let content = i.to_string();
-            let mut new_doc = mk_document! {
+            let mut new_doc = doc! {
                 "_id": i,
                 "content": content,
             };
@@ -711,11 +1122,11 @@ mod tests {
 
         for doc in &doc_collection {
             let key = doc.get("_id").unwrap();
-            let deleted = collection.delete(Some(&mk_document!{
+            let deleted = collection.delete(Some(&doc!{
                 "_id": key.clone(),
             })).unwrap();
             assert!(deleted > 0, "delete nothing with key: {}", key);
-            let find_doc = mk_document! {
+            let find_doc = doc! {
                 "_id": key.clone(),
             };
             let result = collection.find(&find_doc).unwrap();
@@ -746,7 +1157,7 @@ mod tests {
         assert!(collection.insert(&mut doc).unwrap());
 
         let new_id = doc.pkey_id().unwrap();
-        let back = collection.find_one(&mk_document! {
+        let back = collection.find_one(&doc! {
             "_id": new_id,
         }).unwrap().unwrap();
 
